## $Id: browser.py 12662 2015-03-03 14:53:17Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import grok
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from waeup.fceokene.students.interfaces import ICustomStudentOnlinePayment
from waeup.fceokene.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.fceokene.interfaces import MessageFactory as _

PRODUCT_ID = '83'
SITE_NAME = 'fceokene-kofa.waeup.org'
PROVIDER_ACCT = '0026781725'
PROVIDER_BANK_ID = '31'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'FCEOkene'
CURRENCY = '566'
GATEWAY_AMT = 150.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'
POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'
URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = ''
    product_id = PRODUCT_ID

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        provider_amt = 1600.0
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['provider_amt'] = 100 * provider_amt
        # Institution data
        fceokene_split_amt = 1400.0
        xmldict['fceokene_acct'] = "0000000000000"
        xmldict['institution_bank_id'] = '0'
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        if self.context.p_category in ('schoolfee', 'third_semester'):
            self.pay_item_id = '8302'
            if student.current_mode in ('nce_sw','prence',):
                xmldict['institution_acct'] = "1013780934"
                xmldict['institution_bank_id'] = '117'
            elif student.current_mode in ('nce_ft',) and \
                student['studycourse'].current_verdict == 'O':
                xmldict['institution_acct'] = "1013780934"
                xmldict['institution_bank_id'] = '117'
            elif student.current_mode in ('nce_ft',):
                xmldict['institution_acct'] = "1013780934"
                xmldict['institution_bank_id'] = '117'
            elif student.current_mode in ('pd_ft',):
                xmldict['institution_acct'] = "1013780934"
                xmldict['institution_bank_id'] = '117'
            #undergraduate schoolfee added
            elif student.current_mode in ('ug_ft',):
                xmldict['institution_acct'] = "2010952698"
                xmldict['institution_bank_id'] = '8'
            xmldict['fceokene_split'] = 100 * fceokene_split_amt
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt -
                GATEWAY_AMT - fceokene_split_amt)
        elif 'maintenance' in self.context.p_category:
            fceokene_split_amt = 0.0
            provider_amt = 0.0
            self.pay_item_id = '8300'
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - GATEWAY_AMT)
            xmldict['institution_acct'] = "1013780934"
            xmldict['institution_bank_id'] = '117'
        elif self.context.p_category == 'clearance':
            fceokene_split_amt = 0.0
            provider_amt = 0.0
            self.pay_item_id = '8304'
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - GATEWAY_AMT)
            xmldict['institution_acct'] = "2010952698"
            xmldict['institution_bank_id'] = '8'

        # Interswitch amount is not part of the xml data
        if self.context.p_category in ('schoolfee', 'third_semester'):
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="FCEOkene Split" item_amt="%(fceokene_split)d" bank_id="117" acct_num="1013780934" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        self.context.thirdparty_amt = fceokene_split_amt
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '8303'
    product_id = PRODUCT_ID

    def update(self):
        super(CustomInterswitchPageApplicant, self).update()
        xmldict = {}
        # Provider data
        provider_amt = 500.0
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        if getattr(
            self.applicant.__parent__, 'prefix', None) in ('pude', 'putme'):
            xmldict['institution_acct'] = '2010952698'
            xmldict['institution_bank_id'] = '8'
        else:
            xmldict['institution_acct'] = '1013780934'
            xmldict['institution_bank_id'] = '117'
        xmldict['institution_amt'] = 100 * (self.context.amount_auth -
            provider_amt - GATEWAY_AMT)
        xmldict['institution_item_name'] = self.context.p_category
        xmldict['institution_name'] = INSTITUTION_NAME
        # Interswitch amount is not part of the xml data
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return


class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
